/**
  *  \file server/getobj.cc
  *  \brief Server: Implementation of the "GET obj/..." command
  */

#include "server/getobj.h"
#include "cpluslib/ptrvector.h"
#include "server/objout.h"
#include "server/serverio.h"
#include "server/jsonout.h"
#include "game/spec.h"
#include "util/misc.h"
#include "game/global.h"
#include "server/cfgout.h"

static ServerObjectWriter*
createWriter(const string_t obj)
{
    int n;
    if (obj == "shipxy") {
        return new ServerShipxyWriter();
    } else if (obj == "planetxy") {
        return new ServerPlanetxyWriter();
    } else if (obj == "main") {
        return new ServerMainWriter();
    } else if (obj == "player") {
        return new ServerPlayerWriter();
    } else if (obj == "torp") {
        return new ServerTorpWriter();
    } else if (obj == "beam") {
        return new ServerBeamWriter();
    } else if (obj == "engine") {
        return new ServerEngineWriter();
    } else if (obj == "zstorm") {
        return new ServerStormWriter();
    } else if (obj == "zmine") {
        return new ServerMinefieldWriter();
    } else if (obj == "zufo") {
        return new ServerUfoWriter();
    } else if (obj == "truehull") {
        return new ServerTruehullWriter();
    } else if (obj == "zvcr") {
        return new ServerVcrWriter();
    } else if (obj.size() > 4 && obj.compare(0, 4, "hull", 4) == 0 && stringValue(obj.substr(4), n) && n > 0 && n <= getNumHulls()) {
        return new ServerHullWriter(n);
    } else if (obj.size() > 4 && obj.compare(0, 4, "ship", 4) == 0 && stringValue(obj.substr(4), n) && n > 0 && n <= NUM_SHIPS) {
        return new ServerShipWriter(n);
    } else if (obj.size() > 6 && obj.compare(0, 6, "planet", 6) == 0 && stringValue(obj.substr(6), n) && n > 0 && n <= NUM_PLANETS) {
        return new ServerPlanetWriter(n);
    } else if (obj.size() > 3 && obj.compare(0, 3, "msg", 3) == 0 && stringValue(obj.substr(3), n) && n > 0 && n <= getDisplayedTurn().getInbox().getCount()) {
        return new ServerMessageWriter(n);
    } else if (obj.size() > 3 && obj.compare(0, 3, "cfg", 3) == 0 && stringValue(obj.substr(3), n) && n >= 0) {
        return new ServerConfigWriter(n);
    } else {
        return 0;
    }
}

/** Perform "GET obj/..." request.
    \param obj List of objects requested */
void
performGetObject(const string_t obj)
{
    // Collect objects
    ptr_vector<ServerObjectWriter> objs;
    bool fail = false;

    string_t::size_type n = 0, p;
    while ((p = obj.find(',', n)) != string_t::npos && !fail) {
        if (ServerObjectWriter* o = createWriter(obj.substr(n, p-n))) {
            objs.push_back(o);
        } else {
            fail = true;
        }
        n = p+1;
    }
    if (!fail) {
        if (ServerObjectWriter* o = createWriter(obj.substr(n))) {
            objs.push_back(o);
        } else {
            fail = true;
        }
    }

    // Generate output
    if (fail) {
        writeClient("404 Not found");
    } else {
        JsonWriter w;
        writeClient("200 OK");
        w.start();
        for (ptr_vector<ServerObjectWriter>::iterator i = objs.begin(); i != objs.end(); ++i) {
            (*i)->write(w);
        }
        w.end();
        writeClientTerminator();
    }
}
